/* THIS FILE IS AUTOGENERATED - DO NOT EDIT */

#ifndef mozilla_dom_PeerConnectionObserverEnumsBinding_h
#define mozilla_dom_PeerConnectionObserverEnumsBinding_h

#include "js/RootingAPI.h"
#include "jspubtd.h"
#include "mozilla/ErrorResult.h"
#include "mozilla/dom/BindingDeclarations.h"
#include "mozilla/dom/Nullable.h"

namespace mozilla {
namespace dom {

class NativePropertyHooks;
class ProtoAndIfaceCache;

} // namespace dom
} // namespace mozilla

namespace mozilla {
namespace dom {


MOZ_BEGIN_ENUM_CLASS(PCObserverStateType, uint32_t)
  None,
  ReadyState,
  IceConnectionState,
  IceGatheringState,
  SdpState,
  SipccState,
  SignalingState,
  EndGuard_
MOZ_END_ENUM_CLASS(PCObserverStateType)

namespace PCObserverStateTypeValues {
extern const EnumEntry strings[8];
} // namespace PCObserverStateTypeValues


} // namespace dom
} // namespace mozilla

#endif // mozilla_dom_PeerConnectionObserverEnumsBinding_h
