/* This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/. */

#include "nsIDOMApplicationRegistry.idl"

interface nsIDOMDOMRequest;

// This interface is still here for backwards compatibility.
[scriptable, uuid(5bd838b2-cf3d-406e-bbef-f633cf9e68de)]
interface mozIDOMApplicationRegistry2 : mozIDOMApplicationRegistry
{
};
