/* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
/* This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/. */

#include "nsISupports.idl"

interface nsIURI;
interface nsIVariant;

[scriptable, uuid(f816b4df-f733-4dbd-964d-8bfc92a475b2)]
interface nsITaggingService : nsISupports
{
  /**
   * Tags a URL with the given set of tags. Current tags set for the URL
   * persist. Tags in aTags which are already set for the given URL are
   * ignored.
   *
   * @param aURI
   *        the URL to tag.
   * @param aTags
   *        Array of tags to set for the given URL.  Each element within the
   *        array can be either a tag name (non-empty string) or a concrete
   *        itemId of a tag container.
   */
  void tagURI(in nsIURI aURI, in nsIVariant aTags);

  /**
   * Removes tags from a URL. Tags from aTags which are not set for the
   * given URL are ignored.
   *
   * @param aURI
   *        the URL to un-tag.
   * @param aTags
   *        Array of tags to unset.  Pass null to remove all tags from the given
   *        url.  Each element within the array can be either a tag name
   *        (non-empty string) or a concrete itemId of a tag container.
   */
  void untagURI(in nsIURI aURI, in nsIVariant aTags);

  /**
   * Retrieves all URLs tagged with the given tag.
   *
   * @param aTag
   *        tag name
   * @returns Array of uris tagged with aTag.
   */
  nsIVariant getURIsForTag(in AString aTag);

  /**
   * Retrieves all tags set for the given URL.
   *
   * @param aURI
   *        a URL.
   * @returns array of tags (sorted by name).
   */
  void getTagsForURI(in nsIURI aURI,
                     [optional] out unsigned long length,
                     [retval, array, size_is(length)] out wstring aTags);

  /**
   * Retrieves all tags used to tag URIs in the data-base (sorted by name).
   */
  readonly attribute nsIVariant allTags;

  /**
   * Whether any tags exist.
   *
   * @note This is faster than allTags.length, since doesn't need to sort tags.
   */
  readonly attribute boolean hasTags;
};

%{C++

#define TAGGING_SERVICE_CID "@mozilla.org/browser/tagging-service;1"

%}
